"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var three_1 = require("three");
var vertexMaps = {
    west: [0, 1, 2, 3],
    east: [4, 5, 6, 7],
    down: [0, 3, 4, 7],
    up: [2, 1, 6, 5],
    north: [7, 6, 1, 0],
    south: [3, 2, 5, 4]
};
function applyVertexMapRotation(rotation, _a) {
    var _b = __read(_a, 4), a = _b[0], b = _b[1], c = _b[2], d = _b[3];
    return (rotation === 0 ? [a, b, c, d] :
        rotation === 90 ? [b, c, d, a] :
            rotation === 180 ? [c, d, a, b] :
                [d, a, b, c]);
}
function buildMatrix(angle, scale, axis) {
    var a = Math.cos(angle) * scale;
    var b = Math.sin(angle) * scale;
    var matrix = new three_1.Matrix3();
    if (axis === 'x') {
        matrix.set(1, 0, 0, 0, a, -b, 0, b, a);
    }
    else if (axis === 'y') {
        matrix.set(a, 0, b, 0, 1, 0, -b, 0, a);
    }
    else {
        matrix.set(a, -b, 0, b, a, 0, 0, 0, 1);
    }
    return matrix;
}
function rotateCubeCorners(corners, rotation) {
    var origin = new three_1.Vector3()
        .fromArray(rotation.origin)
        .subScalar(8);
    var angle = rotation.angle / 180 * Math.PI;
    var scale = rotation.rescale ? Math.SQRT2 / Math.sqrt(Math.pow(Math.cos(angle || Math.PI / 4), 2) * 2) : 1;
    var matrix = buildMatrix(angle, scale, rotation.axis);
    return corners.map(function (vertex) { return new three_1.Vector3()
        .fromArray(vertex)
        .sub(origin)
        .applyMatrix3(matrix)
        .add(origin)
        .toArray(); });
}
function getCornerVertices(from, to) {
    var _a = __read(from.concat(to).map(function (coordinate) { return coordinate - 8; }), 6), x1 = _a[0], y1 = _a[1], z1 = _a[2], x2 = _a[3], y2 = _a[4], z2 = _a[5];
    return [
        [x1, y1, z1],
        [x1, y2, z1],
        [x1, y2, z2],
        [x1, y1, z2],
        [x2, y1, z2],
        [x2, y2, z2],
        [x2, y2, z1],
        [x2, y1, z1]
    ];
}
function generateDefaultUvs(faceName, _a, _b) {
    var _c = __read(_a, 3), x1 = _c[0], y1 = _c[1], z1 = _c[2];
    var _d = __read(_b, 3), x2 = _d[0], y2 = _d[1], z2 = _d[2];
    return (faceName === 'west' ? [z1, 16 - y2, z2, 16 - y1] :
        faceName === 'east' ? [16 - z2, 16 - y2, 16 - z1, 16 - y1] :
            faceName === 'down' ? [x1, 16 - z2, x2, 16 - z1] :
                faceName === 'up' ? [x1, z1, x2, z2] :
                    faceName === 'north' ? [16 - x2, 16 - y2, 16 - x1, 16 - y1] :
                        [x1, 16 - y2, x2, 16 - y1]);
}
function computeNormalizedUvs(uvs) {
    return uvs.map(function (coordinate, i) {
        return (i % 2 ? 16 - coordinate : coordinate) / 16;
    });
}
var GroupedAttributesBuilder = /** @class */ (function () {
    function GroupedAttributesBuilder(textures) {
        var e_1, _a;
        this.groups = {};
        this.groupMapping = {};
        this.missingGroup = { vertices: [], uvs: [], indices: [] };
        try {
            for (var _b = __values(new Set(Object.values(textures))), _c = _b.next(); !_c.done; _c = _b.next()) {
                var texturePath = _c.value;
                this.groups[texturePath] = { vertices: [], uvs: [], indices: [] };
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        for (var variable in textures) {
            this.groupMapping['#' + variable] = this.groups[textures[variable]];
        }
    }
    GroupedAttributesBuilder.prototype.getContext = function (textureVariable) {
        return this.groupMapping[textureVariable] || this.missingGroup;
    };
    GroupedAttributesBuilder.prototype.getAttributes = function () {
        var _this = this;
        var _a = this.missingGroup, vertices = _a.vertices, uvs = _a.uvs, indices = _a.indices;
        var indexCount = indices.length;
        var groups = [{ start: 0, count: indexCount, materialIndex: 0 }];
        groups.push.apply(groups, __spread(Object.keys(this.groups).sort().map(function (path, i) {
            var group = _this.groups[path];
            var start = indexCount;
            var count = group.indices.length;
            var offset = vertices.length / 3;
            vertices = vertices.concat(group.vertices);
            uvs = uvs.concat(group.uvs);
            indices = indices.concat(group.indices.map(function (index) { return index + offset; }));
            indexCount += count;
            return { start: start, count: count, materialIndex: i + 1 };
        })));
        return { vertices: vertices, uvs: uvs, indices: indices, groups: groups };
    };
    return GroupedAttributesBuilder;
}());
var MinecraftModelGeometry = /** @class */ (function (_super) {
    __extends(MinecraftModelGeometry, _super);
    function MinecraftModelGeometry(model) {
        var e_2, _a;
        var _this = _super.call(this) || this;
        var _b = MinecraftModelGeometry.computeAttributes(model), vertices = _b.vertices, uvs = _b.uvs, indices = _b.indices, groups = _b.groups;
        _this.addAttribute('position', new three_1.Float32BufferAttribute(vertices, 3));
        _this.addAttribute('uv', new three_1.Float32BufferAttribute(uvs, 2));
        _this.setIndex(new three_1.Uint16BufferAttribute(indices, 1));
        try {
            for (var groups_1 = __values(groups), groups_1_1 = groups_1.next(); !groups_1_1.done; groups_1_1 = groups_1.next()) {
                var _c = groups_1_1.value, start = _c.start, count = _c.count, materialIndex = _c.materialIndex;
                _this.addGroup(start, count, materialIndex);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (groups_1_1 && !groups_1_1.done && (_a = groups_1.return)) _a.call(groups_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return _this;
    }
    MinecraftModelGeometry.computeAttributes = function (model) {
        var e_3, _a, e_4, _b;
        var builder = new GroupedAttributesBuilder(model.textures);
        try {
            for (var _c = __values(model.elements), _d = _c.next(); !_d.done; _d = _c.next()) {
                var element = _d.value;
                var from = element.from, to = element.to, rotation = element.rotation;
                var cornerVertices = getCornerVertices(from, to);
                var rotatedVertices = rotation ? rotateCubeCorners(cornerVertices, rotation) : cornerVertices;
                for (var name_1 in element.faces) {
                    var faceName = name_1;
                    var face = element.faces[faceName];
                    var _e = builder.getContext(face.texture), vertices = _e.vertices, uvs = _e.uvs, indices = _e.indices;
                    var i = vertices.length / 3;
                    indices.push(i, i + 2, i + 1);
                    indices.push(i, i + 3, i + 2);
                    try {
                        for (var _f = __values(applyVertexMapRotation(face.rotation || 0, vertexMaps[faceName])), _g = _f.next(); !_g.done; _g = _f.next()) {
                            var index = _g.value;
                            vertices.push.apply(vertices, __spread(rotatedVertices[index]));
                        }
                    }
                    catch (e_4_1) { e_4 = { error: e_4_1 }; }
                    finally {
                        try {
                            if (_g && !_g.done && (_b = _f.return)) _b.call(_f);
                        }
                        finally { if (e_4) throw e_4.error; }
                    }
                    var faceUvs = face.uv || generateDefaultUvs(faceName, from, to);
                    var _h = __read(computeNormalizedUvs(faceUvs), 4), u1 = _h[0], v1 = _h[1], u2 = _h[2], v2 = _h[3];
                    uvs.push(u1, v2);
                    uvs.push(u1, v1);
                    uvs.push(u2, v1);
                    uvs.push(u2, v2);
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c.return)) _a.call(_c);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return builder.getAttributes();
    };
    return MinecraftModelGeometry;
}(three_1.BufferGeometry));
exports.MinecraftModelGeometry = MinecraftModelGeometry;
//# sourceMappingURL=geometry.js.map