"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var three_1 = require("three");
var geometry_1 = require("./geometry");
var loader_1 = require("./loader");
var material_1 = require("./material");
var model_1 = require("./model");
var MinecraftModelLoader = /** @class */ (function (_super) {
    __extends(MinecraftModelLoader, _super);
    function MinecraftModelLoader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MinecraftModelLoader.prototype.load = function (url, onLoad, onProgress, onError) {
        var loader = new three_1.FileLoader(this.manager);
        loader.setPath(this.path);
        loader.setResponseType('json');
        var handleLoad = function (model) {
            try {
                var mesh = new MinecraftModelMesh(model);
                if (onLoad) {
                    onLoad(mesh);
                }
            }
            catch (err) {
                if (onError) {
                    onError(err);
                }
            }
        };
        loader.load(url, handleLoad, onProgress, onError);
    };
    return MinecraftModelLoader;
}(loader_1.AbstractLoader));
exports.MinecraftModelLoader = MinecraftModelLoader;
var MinecraftModelMesh = /** @class */ (function (_super) {
    __extends(MinecraftModelMesh, _super);
    function MinecraftModelMesh(model) {
        var _this = this;
        if (typeof model === 'string') {
            model = JSON.parse(model);
        }
        if (!model_1.isMinecraftModel(model)) {
            throw new Error('Invalid model');
        }
        var geometry = new geometry_1.MinecraftModelGeometry(model);
        var sortedTextures = __spread(new Set(Object.values(model.textures))).sort();
        var mapping = {};
        var materials = sortedTextures
            .map(function (path) { return mapping[path] = new material_1.MinecraftModelMaterial(); });
        _this = _super.call(this, geometry, __spread([material_1.MISSING_TEXTURE_MATERIAL], materials)) || this;
        _this.materialMapping = mapping;
        return _this;
    }
    MinecraftModelMesh.prototype.resolveTextures = function (resolver) {
        for (var path in this.materialMapping) {
            this.materialMapping[path].map = resolver(path);
        }
    };
    return MinecraftModelMesh;
}(three_1.Mesh));
exports.MinecraftModelMesh = MinecraftModelMesh;
//# sourceMappingURL=mesh.js.map